/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unikl.cs.agak.malijan.examples;

import de.unikl.cs.agak.malijan.input.Input;
import de.unikl.cs.agak.malijan.input.InputImpl;
import de.unikl.cs.agak.malijan.input.InputProvider;
import de.unikl.cs.agak.malijan.input.annotations.Choice;
import de.unikl.cs.agak.malijan.input.annotations.Simple;

import java.io.File;
import java.util.Arrays;

/**
 * This dummy input provider is a demo of usable constructor parameter types whose values
 * are given at runtime. (The allowable types are restricted, as they have to be given by
 * the user in some GUI elements).
 *
 * @author Sebastian Wild (s_wild@cs.uni-kl.de)
 */
public class InputProviderParameterTypes implements InputProvider<Integer> {

	public InputProviderParameterTypes(@Simple(label = "int", val = "42") int integer,
		  @Simple(label = "Integer", val = "42") Integer wrapper,
		  @Simple(label = "bool") boolean bool, short s,
		  @Simple(label = "byte", val = "-128") byte b,
		  @Simple(label = "random default value gets second since midnight", val = "random")
		  long secondsSinceMidnight, @Simple(label = "float") float f,
		  @Simple(label = "double", val = "3.142") double d,
		  @Simple(label = "We love unicode", val = "↺") Character arrow,
		  @Choice(label = "predefined", choices = {"-1", "1", "42"}, val = "42") int choice,
		  @Simple(label = "String") String string,
		  @Choice(label = "file", choices = {"/tmp/file-a", "/tmp/file-b"}) File folder,
		  @Simple(label = "char array", val = "{H,a,l,'l','o'}") char[] chars,
		  @Choice(label = "String array",
				 choices = {"{}", "{\"Some string\"}",
						"{\"Escape\\\"ed strings\",\"\",\"Hallo\"}"},
				 val = "{\"not in list\"}") String[] strings) {
		System.out.println("integer = " + integer);
		System.out.println("wrapper = " + wrapper);
		System.out.println("bool = " + bool);
		System.out.println("s = " + s);
		System.out.println("b = " + b);
		System.out.println("secondsSinceMidnight = " + secondsSinceMidnight);
		System.out.println("f = " + f);
		System.out.println("d = " + d);
		System.out.println("arrow = " + arrow);
		System.out.println("choice = " + choice);
		System.out.println("string = " + string);
		System.out.println("folder = " + folder);
		System.out.println("chars = " + Arrays.toString(chars));
		System.out.println("strings = " + Arrays.toString(strings));
	}

	@Override public boolean hasNext() throws Throwable {
		return false;
	}

	@Override public Input<Integer> next() throws Throwable {
		return new InputImpl<Integer>(0, 0);
	}

	@Override public long size() throws Throwable {
		return 0;
	}
}
