/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.examples;

import java.util.Arrays;

import static de.unikl.cs.agak.malijan.annotation.Annotations.consumeInput;
import static de.unikl.cs.agak.malijan.annotation.Annotations.produceCost;

/**
 * This class implements straight insertion sort as it can be found in »An
 * Introduction to the Analysis of Algorithms« by Robert Sedgewick and Philippe
 * Flajolet (Program 6.2 on page 335)
 *
 * @author Sebastian Wild
 */
public class InsertionSort {

   public static void main(String[] args) {
      int[] a = new int[args.length];
      for (int i = 0; i < args.length; ++i) {
         a[i] = Integer.parseInt(args[i]);
      }
      sort(a);
      System.out.println(Arrays.toString(a));
   }

   public static void sort(int[] a) {
      consumeInput(1); // {a[0]} is already sorted
      for (int i = 1; i < a.length; ++i) {
         int v = a[i];
         int j = i - 1;
         produceCost("comp", 1); // first time in while
         while (a[j] > v) {
            a[j + 1] = a[j];
            produceCost("move", 1);
            --j;
            if (j == -1) {
               break;
            }
            produceCost("comp", 1); // while cond
         }
         a[j + 1] = v;
         produceCost("move", 1);
         consumeInput(1); // v just arrived at final pos
      }
   }
}
