/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */

package de.unikl.cs.agak.malijan.examples;

import de.unikl.cs.agak.malijan.input.Input;
import de.unikl.cs.agak.malijan.input.InputImpl;
import de.unikl.cs.agak.malijan.input.InputProvider;
import de.unikl.cs.agak.malijan.input.annotations.Label;
import de.unikl.cs.agak.malijan.input.annotations.Simple;

import javax.swing.*;

/**
 * A minimal example of a custom input provider, it queries the user for
 * @author Sebastian Wild (s_wild@cs.uni-kl.de) */

@Label("Interactively ask user for inputs. Input AND Input size are given integer")
public class InteractiveInts implements InputProvider<Integer> {

	private int left;
	private final int size;

	public InteractiveInts(@Simple(label = "number of inputs",val = "30") final int left) {
		this.left = left;
		this.size = left;
	}

	@Override public boolean hasNext() throws Throwable {
		return left > 0;
	}

	@Override public Input<Integer> next() throws Throwable {
		if (--left < 0) return null;
		final String input = JOptionPane.showInputDialog("Enter number");
		final int i = Integer.parseInt(input);
		return new InputImpl<Integer>(i,i);
	}

	@Override public long size() throws Throwable {
		return size;
	}
}
