/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.examples;

import de.unikl.cs.agak.malijan.annotation.Algorithm;
import de.unikl.cs.agak.malijan.annotation.Analyzed;

import static de.unikl.cs.agak.malijan.annotation.Annotations.consumeInput;
import static de.unikl.cs.agak.malijan.annotation.Annotations.produceCost;

/**
 * A collection of simple examples to test MaLiJAn
 *
 * @author Sebastian Wild <s_wild.cs.uni-kl.de>
 */
public class LittleExamples {

	public static void nothing() {
		
	}

	public LittleExamples() {
		
	}

   public static void constantPs(int n) {
      for (int i = 0; i < 4; ++i) {
         for (int j = 0; j < n; ++j) {
            produceCost("", 1);
         }
      }
   }

   public static void ThreeToTheN(int n) {
      if (n == 0) return;
      produceCost("", 1);
      for (int i = 0; i < 3; ++i)
         ThreeToTheN(n - 1);
   }

   public static void nSquared(int n) {
      for (int i = 0; i < n; ++i) {
         for (int j = 0; j < n; ++j) {
            produceCost("", 1);
//	         System.out.println("i = " + i);
         }
         consumeInput(1);
      }
   }

	@Algorithm
	public static void halfNSquared(int n) {
      for (int i = 0; i < n; ++i) {
         for (int j = 0; j < i; ++j) {
            produceCost("", 1);
         }
         consumeInput(1);
      }
   }

   public static void logN(int n) {
      for (int i = 0; i < n; ++i) {
         consumeInput(1);
         if (isPowerOfTwo(i)) {
            produceCost("", 1);
         }
      }
   }

   public static int faculty(int n) {
      consumeInput(1);
      produceCost("", 2);
      return n == 1 ? 1 : n * faculty(n - 1);
   }

   public static void rec(int n) {
      produceCost("", 1);
      rec(n / 2);
   }

	@Analyzed
   private static boolean isPowerOfTwo(int n) {
      int p = 1;
      while (p < n) {
         p *= 2;
      }
      return p == n;
   }


   public static void halfN(int n) {
      if (n <= 0) return;
      halfN(n - 2);
   }

	public static void compactifiedListsOfBasicBlocks(int n) {
		produceCost("cost",1);
		if (n%2 == 1) { produceCost("uneven",1); }
		produceCost("cost",1);
		if (n%2 == 1) { produceCost("uneven",1); }
		produceCost("cost",1);
		for (; n > 0; n /= 7) produceCost("cost", 7);
		if (n%2 == 1) { produceCost("uneven",1); }
		produceCost("cost", 42);
	}


	public static void caughtException(int n) {
		try {
			if (n % 2 == 0) throw new Exception();
			produceCost("even",1);
		} catch (Exception e) {
			produceCost("odd",1);
		}
	}

	public static void main(String[] args) {
		nSquared(10);
	}
	
	public static void runtimeError(final int n) {
    if ( (n + 1) % 5 == 0 ) {
	    throw new StackOverflowError("Test error");
    }
	}

  public static void uncaughtRuntimeException(final int n) {
    if ( (n + 1) % 5 == 0 ) {
      throw new NullPointerException("Test error");
    }
  }

  public static void uncaughtException(final int n) throws Exception {
    if ( (n + 1) % 5 == 0 ) {
      throw new Exception("Test error");
    }
  }

  public static void soutPrints(final int n) {
    for ( int i=0; i<n; i++) {
      if ( (n + 1) % 5 == 0 ) {
        System.out.println("test");
      }
    }
  }

  public static void serrPrints(final int n) {
    for ( int i=(n + 1); i<n; i++) {
      if ( (n + 1) % 5 == 0 ) {
        System.err.println("test");
      }
    }
  }
}
