/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.examples;

import java.util.Arrays;

import static de.unikl.cs.agak.malijan.annotation.Annotations.produceCost;

/**
 * This class contains quicksort as an example algorithm. The main method is
 * {@link #quicksort(int[])}, working on int[]s
 * @author Sebastian Wild */
public class SedgewicksQuicksort {

	public static void main(String[] args) {
		int[] a = new int[]{6, 78, 3, 4, 1, 43, 5, 6};
		quicksortInlined(a);
		System.out.println(Arrays.toString(a));
	}

	public static void quicksort(int[] a) {
		quicksort(a, 0, a.length - 1);
	}

	public static void quicksortInlined(int[] a) {
		quicksortInlined(a, 0, a.length - 1);
	}


	public static void quicksort(int[] a, int l, int r) {
		if (r <= l) {
			return;
		}

		int i = partition(a, l, r);
		quicksort(a, l, i - 1);
		quicksort(a, i + 1, r);
	}

	public static int partition(int[] a, int l, int r) {
		int i = l - 1, j = r;
		int v = a[r];
		for (; ; ) {
			while (a[++i] < v & produceCost("comparison", 1)) ;
			while (v < a[--j] & produceCost("comparison", 1)) {
				if (j == i) {
					break;
				}
			}

			if (i >= j) {
				break;
			}

			swap(a, i, j);
		}

		swap(a, i, r);
		return i;
	}

	/**
	 * swaps to enties in the array a
	 *
	 * @param a
	 * @param index1
	 * @param index2
	 */
	public static void swap(int[] a, int index1, int index2) {
		if (index1 == index2) {
			return;
		}

		produceCost("swap", 1);
		int tmp = a[index2];
		a[index2] = a[index1];
		a[index1] = tmp;
	}





	public static void quicksortInlined(int[] a, int l, int r) {
		if (r <= l) {
			return;
		}
		int i = l - 1, j = r;
		int pivot = a[r];
		for (; ; ) {
			while (a[++i] < pivot & produceCost("comparison", 1)) ;
			while (pivot < a[--j] & produceCost("comparison", 1)) {
				if (j == i) {
					break;
				}
			}

			if (i >= j) {
				break;
			}

			if (i != j) {
				produceCost("swap", 1);
				int tmp = a[j];
				a[j] = a[i];
				a[i] = tmp;
			}
		}

		if (i != r) {
			produceCost("swap", 1);
			int tmp = a[r];
			a[r] = a[i];
			a[i] = tmp;
		}
		quicksortInlined(a, l, i - 1);
		quicksortInlined(a, i + 1, r);
	}


}
