/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.examples;

import java.util.Arrays;

import static de.unikl.cs.agak.malijan.annotation.Annotations.consumeInput;
import static de.unikl.cs.agak.malijan.annotation.Annotations.produceCost;

/**
 * Simple sorting algorithm to demonstrate MaLiJAn
 *
 * @author Sebastian Wild
 */
public class SelectionSort {

   /**
    * Sorts the given array a ascendingly.
    *
    * @param a
    */
   public static int[] selectionsort(int[] a) {
      System.out.println("Sorting " + Arrays.toString(a));
      for (int end = a.length; end > 0; --end) {
         // find maximum
         int curMaxIndex = findMaxIndex(a, 0, end);

         // swap curMax with a[end]
         swap(a, curMaxIndex, end - 1);
         consumeInput(1); // one entry (a[end]) now in its final place
      }

      System.out.println(Arrays.toString(a));
      return a;
   }

   /**
    * swaps to enties in the array a
    *
    * @param a
    * @param index1
    * @param index2
    */
   public static void swap(int[] a, int index1, int index2) {
      if (index1 == index2) return;
      produceCost("swap", 1);
      int tmp = a[index2];
      produceCost("array access", 1);
      a[index2] = a[index1];
      produceCost("array access", 1);
      a[index1] = tmp;
      produceCost("array access", 1);
   }

   /**
    * Finds the index of a maximal entry in {@code a[start] ... a[end-1]
    *
    * @param a
    * @param start
    * @param end
    * @return index
    */
   public static int findMaxIndex(int[] a, int start, int end) {
      int curMax = a[start];
      produceCost("array access", 1);
      int curMaxIndex = start;
      for (int i = start + 1; i < end; ++i) {
         // cost for condition
         produceCost("key comparison", 1);
         produceCost("array access", 1);
         if (a[i] > curMax) {
            curMax = a[i];
            produceCost("array access", 1);
            curMaxIndex = i;
         }
      }
      return curMaxIndex;
   }
}
