/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.input;

import de.unikl.cs.agak.malijan.util.Viewable;

import java.io.Serializable;

/**
 * Value object containing one input of type T and its assigned size.
 * <p/>
 * Implementations must provide value semantics based on {@link #id()}.
 * <em>Recall:</em> If inputs are randomly generated, there may be two inputs of same size
 * and content; yet we would like to treat them as two independent input. On the other
 * hand, as instances of this class are passed via RMI, a sensible notion of equality
 * should not be based in reference comparisons.
 * @param <T> The type of underlying input values
 *
 * @author Sebastian Wild (s_wild@cs.uni-kl.de)
 */
//@ValueType
//@Immutable
public interface Input<T extends Serializable> extends Serializable, Viewable {
  /**
   * The size of input {@code data} wrt algorithm complexity.
   * @return some size measure
   */
   long size();

  /**
   *
   * @return the wrapped algorithm input value
   */
   T data();

   /** @return internal id for this input, unique wrt its {@link InputProvider}. */
   long id();
}
