/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.input.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * Used to provide more information about parameters of constructors.
 * Other than {@link Simple}, {@code Choice} restricts the parameter to be one of a finite list
 * of values.
 * @author Raphael Reitzig, 09.2011
 */
@Target(ElementType.PARAMETER)
@Retention(RetentionPolicy.RUNTIME)
public @interface Choice {
  /**
   * @return the label the annotated parameter should be represented with
   */
  String label() default "PARAMETER_INDEX";

  /**
   * @return default value of the annotated parameter
   */
  String val() default "NONE";

  /**
   * @return array with legal values for the annotated parameter
   */
  String[] choices();
}
