/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unikl.cs.agak.malijan.input.annotations;

import de.unikl.cs.agak.malijan.input.Input;
import de.unikl.cs.agak.malijan.input.InputImpl;
import de.unikl.cs.agak.malijan.input.InputProvider;

/**
 * A minimal example for an implementation of {@link InputProvider} using the
 * annotations in this package.
 *
 * @author Raphael Reitzig, 09.2011
 */
@Label("2-Step Counter (Example)")
public class Example implements InputProvider<Integer> {
  private final int n;
  private int i;

   public Example(@Simple(label = "Maximum", val = "10") final int n,
        @Choice(label = "Kind", val = "even", choices = {"even", "odd"})
        final String kind) {
      this.n = n;
      this.i = "even".equals(kind) ? -2 : -1;
   }

   @Override
  public boolean hasNext() {
    return i < n;
  }

  @Override
  public Input<Integer> next() {
    i += 2;
    return hasNext() ? new InputImpl<Integer>(i, i) : null;
  }

  @Override
  public long size() {
    return n;
  }
}
