/*
 * This file is part of MaLiJAn-GUI.
 *
 * MaLiJAn-GUI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MaLiJAn-GUI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MaLiJAn-GUI.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * This package contains annotations intended to guide user interface code to representing and
 * initialising {@link InputProvider}s in a proper way. In particular, there are the following
 * annotations:
 * <ul>
 *   <li>{@link Label}: Annotates classes and provides a label suitable for user interface usage.</li>
 *   <li>{@link Simple}: Annotates constructor parameters and assigns labels and default values
 *                       (as string representations). For {@code int} and {@code long} parameters,
 *                       special value "random" is allowed which will ask input provider
 *                       configuration to suggest an arbitrary value.</li>
 *   <li>{@link Choice} Annotates constructor parameters and assigns labels, default values
 *                      (as string representations) and a lists of allowed values.</li>
 * </ul>
 * Note that {@link InputProvider}s not using these annotations might be picked up by the UI, but even if
 * it will not be able to represent them nicely.
 * @see Example A simple example
 * @author Raphael Reitzig, 09.2011
 */
package de.unikl.cs.agak.malijan.input.annotations;

import de.unikl.cs.agak.malijan.input.InputProvider;